# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

from django.db import models
from options.settings import DEFAULT_EXCLUDE_USER_OPTIONS

class OptionManager(models.Manager):
    """Manager for options."""

    def get_value(self, name, default=None):
        """Gets the value with the proper type."""
        try:
            option = self.model.objects.get(name=name)
            return option.get_value()
        except self.model.DoesNotExist:
            return default


class UserOptionManager(models.Manager):
    """Manager to handle user's custom options."""

    def filter_user_customizable(self):
        """Returns option that the user can customize himself."""
        return self.exclude(name__in=DEFAULT_EXCLUDE_USER_OPTIONS)

    def get_value(self, name, user=None, default=None):
        """Gets the value with the proper type."""
        from options.models import Option

        if user is None:
            return Option.objects.get_value(name=name, default=default)
        try:
            option = self.model.objects.get(user=user, name=name)
            return option.get_value()
        except self.model.DoesNotExist:
            return Option.objects.get_value(name=name, default=default)



# Generated by Django 2.1 on 2018-10-02 07:54

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('options', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('public_name', models.CharField(db_index=True, max_length=255, verbose_name='Public name of the parameter')),
                ('type', models.PositiveIntegerField(choices=[(0, 'Float'), (1, 'Integer'), (2, 'String')], default=2)),
                ('value', models.CharField(blank=True, default=None, max_length=256, null=True, verbose_name='Value')),
                ('is_list', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=255, verbose_name='Parameter')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='options', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['public_name'],
            },
        ),
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ['public_name']},
        ),
        migrations.AlterUniqueTogether(
            name='useroption',
            unique_together={('user', 'name')},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 15:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=255, unique=True, verbose_name='Parameter')),
                ('public_name', models.CharField(db_index=True, max_length=255, verbose_name='Public name of the parameter')),
                ('type', models.PositiveIntegerField(choices=[(0, 'Float'), (1, 'Integer'), (2, 'String')], default=2)),
                ('value', models.CharField(blank=True, default=None, max_length=256, null=True, verbose_name='Value')),
                ('is_list', models.BooleanField(default=False)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import logging
import six
from django.apps import AppConfig
from django.db.models.signals import post_migrate
from django.db.utils import IntegrityError

from options.settings import DEFAULT_OPTIONS

logger = logging.getLogger(__name__)


def create_default_options(sender, **kwargs):
    """Creates the defaults configuration options if they don't exists."""
    from options.models import Option
    for key, data in six.iteritems(DEFAULT_OPTIONS):
        if not Option.objects.filter(name=key).exists():
            try:
                Option.objects.create(name=key, **data)
            except IntegrityError:
                logger.warning("Option '%s' already installed" % key)


class ConfigurationsConfig(AppConfig):
    name = "options"

    def ready(self):
        """Connects signals with their managers."""
        post_migrate.connect(create_default_options)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

from django.conf import settings


# Set on settings the default options for this project. These will be created
# on the post migrate signal handler.
# Sample:
#
# CONFIGURATION_DEFAULT_OPTIONS = {
#     "sold_out": {
#         "value": 0,
#         "type": INT,
#         "public_name": "Sets tickets as sold out"
#     },
# }
#
DEFAULT_OPTIONS = getattr(settings, "CONFIGURATION_DEFAULT_OPTIONS", {})

# Set the list of options that the user can't customize.
DEFAULT_EXCLUDE_USER_OPTIONS = getattr(settings, "EXCLUDE_USER_OPTIONS", tuple())


# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

from django import get_version
from django.utils.translation import ugettext_lazy as _


FLOAT, INT, STRING = (0, 1, 2)
TYPE_CHOICES = (
    (FLOAT, _("Float")),
    (INT, _("Integer")),
    (STRING, _("String")),
)

default_app_config = 'options.apps.ConfigurationsConfig'

VERSION = (1, 0, 0, 'final', 0)

__version__ = get_version(VERSION)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django.contrib import admin

from options.models import Option


@admin.register(Option)
class OptionAdmin(admin.ModelAdmin):
    """Manage configuration options."""

    list_display = ['public_name', 'value']
    search_fields = ['public_name', 'name']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import six
from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from options import STRING, TYPE_CHOICES, INT, FLOAT
from options.managers import OptionManager, UserOptionManager

@python_2_unicode_compatible
class BaseOption(models.Model):
    """Base model for system options and configurations."""

    name = models.CharField(
        verbose_name=_("Parameter"),
        max_length=255,
        unique=True,
        db_index=True
    )
    public_name = models.CharField(
        verbose_name=_("Public name of the parameter"),
        max_length=255,
        unique=False,
        db_index=True
    )
    type = models.PositiveIntegerField(
        choices=TYPE_CHOICES,
        default=STRING
    )
    value = models.CharField(
        null=True,
        blank=True,
        default=None,
        max_length=256,
        verbose_name=_("Value")
    )
    is_list = models.BooleanField(default=False)

    class Meta:
        abstract = True

    def __str__(self):
        return "%s" % self.public_name

    def get_value(self):
        """Gets the value with the proper type."""
        converter = {
            INT: int,
            FLOAT: float,
            STRING: six.text_type
        }
        if not self.is_list:
            return converter.get(self.type, six.text_type)(self.value)
        else:
            values = self.value.split(",")
            return list(map(lambda item: converter.get(self.type, six.text_type)(item), values))



class Option(BaseOption):
    """System options and configurations."""

    objects = OptionManager()

    class Meta:
        ordering = ["public_name"]



class UserOption(BaseOption):
    """Custom option for a user."""
    
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="options", on_delete=models.CASCADE)
    name = models.CharField(
        verbose_name=_("Parameter"),
        max_length=255,
    )

    objects = UserOptionManager()

    class Meta:
        unique_together = ["user", "name"]
        ordering = ["public_name"]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, division, absolute_import

import os

from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

with open(os.path.join(os.path.dirname(__file__), 'HISTORY.rst')) as history:
    HISTORY = history.read().replace('.. :changelog:', '')

# Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# Dynamically calculate the version based on belt.VERSION.
version = __import__('options').__version__


setup(
    name='django-simple-options',
    version=version,
    packages=[
        'options.management.commands',
        'options.migrations',
        'options',
    ],
    include_package_data=True,
    license='MIT License',
    description='Simple app to add configuration options to a Django project',
    long_description=README + '\n\n' + HISTORY,
    url='https://github.com/marcosgabarda/django-options',
    author='Marcos Gabarda',
    author_email='hey@marcosgabarda.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Utilities',
    ],
    install_requires=[
        'django>=1.9',
        'six>=1.0',
    ],
)
